@@ -1,6 +1,6 @@
 /* 
    cadaver, command-line DAV client
-   Copyright (C) 1999-2001, Joe Orton <joe@orton.demon.co.uk>, 
+   Copyright (C) 1999-2001, Joe Orton <joe@manyfish.co.uk>, 
    except where otherwise indicated.
                                                                      
    This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#include <config.h>
+#include "config.h"
 
 #include <sys/types.h>
 
@@ -27,8 +27,8 @@
 
 #include <stdio.h>
 #include <ctype.h>
-
 #include <signal.h>
+#include <time.h>
 
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
@@ -72,6 +72,7 @@
 #include "cmdline.h"
 #include "commands.h"
 #include "options.h"
+#include "utils.h"
 
 #define DEFAULT_NAMESPACE "http://webdav.org/cadaver/custom-properties/"
 
@@ -97,7 +98,6 @@ char *path, /* current working collection */
 
 int tolerant; /* tolerate DAV-enabledness failure */
 
-int in_curses = 0;
 int use_expect = 0;
 
 int have_connection = 0; /* true when we are connected to the server */
@@ -114,34 +114,20 @@ static enum out_state {
 
 /* Protoypes */
 
-static void usage(void);
-static void parse_args(int argc, char *argv[]);
-static char *read_command(void);
-
 static RETSIGTYPE quit_handler(int signo);
 
 static void close_connection(void);
 static void open_connection(const char *url);
 
-static void init_netrc(void);
-static void init_rcfile(void);
 static void execute_version(void);
-static void init_readline(void);
-static void init_options(void);
 
 static void transfer_progress(void *ud, off_t progress, off_t total);
 static void connection_status(void *ud, ne_conn_status status, 
 			      const char *info);
-static int supply_creds_server(void *userdata, const char *realm,
-			       char **username, char **password);
-static int supply_creds_proxy(void *userdata, const char *realm,
-			      char **username, char **password);
-
-#ifdef HAVE_FILENAME_COMPLETION_FUNCTION
-
-static char **completion(char *text, int start, int end);
-
-#endif
+static int supply_creds_server(void *userdata, const char *realm, int attempt,
+			       char *username, char *password);
+static int supply_creds_proxy(void *userdata, const char *realm, int attempt,
+			      char *username, char *password);
 
 #define CMD_VARY 9999
 
@@ -155,7 +141,7 @@ const struct command commands[] = {
 #define UO1(x,c,h) { cmd_##x, false, 0, 1, parmscope_none, execute_##x,c,h }
 #define C2M(x,c,h) { cmd_##x, true, 2, CMD_VARY, parmscope_remote, multi_##x,c,h }
 #define C1M(x,c,h) { cmd_##x, true, 1, CMD_VARY, parmscope_remote, multi_##x,c,h }
-    { cmd_ls, true, 0, 1, parmscope_none, execute_ls, 
+    { cmd_ls, true, 0, 1, parmscope_remote, execute_ls, 
       N_("ls [path]"), N_("List contents of current [or other] collection") },
     C1(cd, N_("cd path"), N_("Change to specified collection")),
     { cmd_pwd, true, 0, 0, parmscope_none, execute_pwd,
@@ -196,6 +182,9 @@ const struct command commands[] = {
     { cmd_propget, true, 1, 2, parmscope_none, execute_propget,
       N_("propget res [propname]"), 
       N_("Retrieve properties of resource") },
+    { cmd_propdel, true, 2, 2, parmscope_none, execute_propdel,
+      N_("propdel res propname"), 
+      N_("Delete property from resource") },
     { cmd_propset, true, 3, 3, parmscope_none, execute_propset,
       N_("propset res propname value"),
       N_("Set property on resource") },
@@ -232,7 +221,7 @@ const struct command commands[] = {
 #undef C1M
 };    
 
-static void usage()
+static void usage(void)
 {
     printf(_(
 "Usage: %s [OPTIONS] http://hostname[:port]/path\n"
@@ -267,8 +256,7 @@ static void close_connection(void)
  * collection. */
 int set_path(const char *newpath)
 {
-    int is_coll;
-    is_coll = is_collection(newpath);
+    int is_coll = (getrestype(newpath) == resr_collection);
     if (is_coll || tolerant) {
 	if (!is_coll) {
 	    dav_collection = false;
@@ -375,7 +363,7 @@ static void open_connection(const char *url)
 
     session = ne_session_create();
     ssl_context = sock_create_ssl_context();
-/*    http_redirect_register(session, NULL, redirect_notify, NULL); */
+    ne_redirect_register(session, NULL, redirect_notify, NULL);
 
     init_locking(lockstore);
     
@@ -716,45 +704,125 @@ static void init_rcfile(void)
     free(rcfile);
 }
 
-#ifdef HAVE_FILENAME_COMPLETION_FUNCTION
 
-static char **completion(char *text, int start, int end)
+#ifdef HAVE_LIBREADLINE
+
+#define COMPLETION_CACHE_EXPIRE 10 /* seconds */
+
+#ifndef HAVE_RL_COMPLETION_MATCHES
+/* readline <4.2 compatibility. */
+#define rl_completion_matches completion_matches
+#define rl_filename_completion_function filename_completion_function
+#endif
+
+static char *remote_completion(const char *text, int state)
+{
+    static struct resource *reslist, *current;
+    static int len;
+    static time_t last_fetch;
+    static char *last_path;
+
+    char *name;
+    
+    if (state == 0) {
+	/* Check to see if we should refresh the dumb cache.
+	 * or, initialize the local cache of remote filenames
+	 * The remote resource list persists until a completion
+	 * in a new context is requested or the cache expires.
+	 */
+
+	/* TODO get cache expire time from config, currently from cadaver.h
+	 * TODO cache and fetch on deep/absolute paths: (path: /a/b/, text: c/d)
+	 */
+	if (last_fetch < (time(NULL) - COMPLETION_CACHE_EXPIRE) 
+	    || !last_path 
+	    || strcmp(path, last_path) != 0) {
+
+	    if (last_path != NULL) {
+		free(last_path);
+	    }
+
+	    if (reslist != NULL) { 
+		free_resource_list(reslist); 
+	    }
+
+	    /* Hide the connection status */
+	    ne_set_status(session, NULL, NULL);
+	    if (fetch_resource_list(session, path, 1, 0, &reslist) != NE_OK) {
+		reslist = NULL;
+	    }
+	    /* Restore the session connection printing */
+	    ne_set_status(session, connection_status, NULL);
+
+	    last_path = ne_strdup(path);
+	}
+
+	current = reslist;
+	len = strlen(text);
+	time(&last_fetch);
+    }
+
+    while (current) {
+	/* Massage the absolute URI to a URI relative to our path */
+	/* Copy & paste & search & replace from ls.c */
+	if (uri_has_trailing_slash(current->uri)) {
+	    current->uri[strlen(current->uri)-1] = '\0';
+	}
+
+	name = strrchr(current->uri, '/');
+	if (name != NULL && strlen(name+1) > 0) {
+	    name++;
+	} else {
+	    name = current->uri;
+	}
+	name = uri_unescape(name);
+
+	if (strncmp(text, name, len) == 0) {
+	    current = current->next;
+	    return ne_strdup(name);
+	}
+
+	current = current->next;
+    }
+    
+    return NULL;
+}
+
+static char **completion(const char *text, int start, int end)
 {
     char **matches = NULL;
-    
+    char *sep = strchr(rl_line_buffer, ' ');
+
     if (start == 0) {
-	matches = completion_matches(text, command_generator);
-    } else {
-	char *sep = strchr(rl_line_buffer, ' ');
-	/* Find command name, so we can work out whether to do local
-	 * or remote completion */
-	if (sep != NULL) {
-	    char *cname = ne_strndup(rl_line_buffer, sep - rl_line_buffer);
-	    const struct command *cmd;
-	    cname[sep - rl_line_buffer] = '\0';
-	    cmd = get_command(cname);
-	    if (cmd != NULL) { 
-		switch (cmd->scope) {
-		case parmscope_none:
-		    break;
-		case parmscope_local:
-		    matches = completion_matches(text, filename_completion_function);
-		    break;
-		case parmscope_option:
-		    /* TODO */
-		    break;
-		case parmscope_remote:
-		    /* TODO */
-		    break;
-		}
-	    }
-	    free(cname);
-	}		    
+	matches = rl_completion_matches(text, command_generator);
     }
+    else if (sep != NULL) {
+	char *cname = ne_strndup(rl_line_buffer, sep - rl_line_buffer);
+	const struct command *cmd;
+	cname[sep - rl_line_buffer] = '\0';
+	cmd = get_command(cname);
+	if (cmd != NULL) { 
+	    switch (cmd->scope) {
+	    case parmscope_none:
+		break;
+	    case parmscope_local:
+		matches = rl_completion_matches(text, 
+						rl_filename_completion_function);
+		break;
+	    case parmscope_option:
+		/* TODO */
+		break;
+	    case parmscope_remote:
+		matches = rl_completion_matches(text, remote_completion);
+		break;
+	    }
+	}
+	free(cname);
+    }		    
     return matches;
 }
 
-#endif /* HAVE_FILENAME_COMPLETION_FUNCTION */
+#endif /* HAVE_LIBREADLINE */
 
 void output(enum output_type t, const char *fmt, ...)
 {
@@ -786,13 +854,11 @@ void output(enum output_type t, const char *fmt, ...)
     }
 }
 
-static void init_readline()
+static void init_readline(void)
 {
 #ifdef HAVE_LIBREADLINE
     rl_readline_name = "cadaver";
-#ifdef HAVE_FILENAME_COMPLETION_FUNCTION
-    rl_attempted_completion_function = (CPPFunction *)completion;
-#endif /* HAVE_FILENAME_COMPLETION_FUNCTION */
+    rl_attempted_completion_function = completion;
 #endif /* HAVE_LIBREADLINE */
 }
 
@@ -814,7 +880,7 @@ char *readline(const char *prompt)
 }
 #endif
 
-static void init_options()
+static void init_options(void)
 {
     char *lockowner, *tmp;
     char *user = getenv("USER"), *hostname = getenv("HOSTNAME");
@@ -829,6 +895,9 @@ static void init_options()
 
     set_option(opt_editor, NULL);
     set_option(opt_namespace, ne_strdup(DEFAULT_NAMESPACE));
+    set_bool_option(opt_overwrite, 1);
+    lockdepth = NE_DEPTH_INFINITE;
+    lockstore = ne_lockscope_exclusive;
 
     /* This is what Markus Kahn says we should do. */
     if ((tmp = getenv("LC_ALL")) ||
@@ -906,6 +975,9 @@ int main(int argc, char *argv[])
 
 static void connection_status(void *ud, ne_conn_status status, const char *info)
 {
+    if (get_bool_option(opt_quiet)) {
+	return;
+    }
     switch (out_state) {
     case out_none:
 	switch (status) {
@@ -1023,7 +1095,6 @@ static void pretty_progress_bar(off_t progress, off_t total)
 
 static void transfer_progress(void *ud, off_t progress, off_t total)
 {
-    if (in_curses) return;
     switch (out_state) {
     case out_none:
     case out_incommand:
@@ -1052,9 +1123,10 @@ static void transfer_progress(void *ud, off_t progress, off_t total)
 }
 
 static int supply_creds(const char *prompt, const char *realm, const char *hostname,
-			char **username, char **password)
+			char *username, char *password)
 {
-    if (in_curses) return -1;
+    char *tmp;
+
     switch (out_state) {
     case out_transfer_pretty:
 	putchar('\n');
@@ -1071,21 +1143,32 @@ static int supply_creds(const char *prompt, const char *realm, const char *hostn
 	break;
     }
     printf(prompt, realm, hostname);
-    *username = readline(_("Username: "));
-    if (*username == NULL) {
-	/* TODO: UI is crap here */
+    
+    tmp = readline(_("Username: "));
+    if (tmp == NULL) {
 	printf(_("\rAuthentication aborted!\n"));
 	return -1;
-    } else {
-    	char *pass = fm_getpassword(_("Password: "));
-	if (pass == NULL) {
-	    printf(_("Authentication aborted!\n"));
-	    free(*username);
-	    return -1;
-	}
-	*password = ne_strdup(pass);
-	memset(pass, 0, PASSWORDLEN);
+    } else if (strlen(tmp) >= NE_ABUFSIZ) {
+	printf(_("\rUsername too long (>%d)\n"), NE_ABUFSIZ);
+	free(tmp);
+	return -1;
     }
+
+    strcpy(username, tmp);
+    free(tmp);
+
+    tmp = fm_getpassword(_("Password: "));
+    if (tmp == NULL) {
+	printf(_("Authentication aborted!\n"));
+	return -1;
+    } else if (strlen(tmp) >= NE_ABUFSIZ) {
+	printf(_("\rPassword too long (>%d)\n"), NE_ABUFSIZ);
+	memset(tmp, strlen(tmp), 0);
+	return -1;
+    }
+    
+    strcpy(password, tmp);
+	
     switch (out_state) {
     case out_transfer_start:
     case out_incommand:
@@ -1102,13 +1185,16 @@ static int supply_creds(const char *prompt, const char *realm, const char *hostn
     return 0;
 }
 
-static int supply_creds_server(void *userdata, const char *realm,
-			       char **username, char **password)
+static int supply_creds_server(void *userdata, const char *realm, int attempt,
+			       char *username, char *password)
 {
+    if (attempt > 1)
+	return -1;
+
     /* Pass back netrc values if we have them. */
     if (server_username != NULL && server_password != NULL) {
-	*username = ne_strdup(server_username);
-	*password = ne_strdup(server_password);
+	strncpy(username, server_username, NE_ABUFSIZ);
+	strncpy(password, server_password, NE_ABUFSIZ);
 	return 0;
     }
 
@@ -1117,9 +1203,12 @@ static int supply_creds_server(void *userdata, const char *realm,
 	server_hostname, username, password);
 }
 
-static int supply_creds_proxy(void *userdata, const char *realm,
-			       char **username, char **password) 
+static int supply_creds_proxy(void *userdata, const char *realm, int attempt,
+			      char *username, char *password) 
 {
+    if (attempt > 1)
+	return -1;
+
     return supply_creds(
 	_("Authentication required for %s on proxy server `%s':\n"), realm,
 	proxy_hostname, username, password);
