@@ -56,7 +56,7 @@ enum command_id {
     cmd_less, cmd_cat, cmd_lpwd, cmd_lcd, cmd_lls, cmd_mput, cmd_mget,
     cmd_echo, cmd_set, cmd_unset, cmd_rmcol, cmd_lock, cmd_unlock,
     cmd_steal, cmd_discover, cmd_showlocks, cmd_propedit, cmd_propnames,
-    cmd_propget, cmd_propset, cmd_chexec, cmd_edit, cmd_logout,
+    cmd_propget, cmd_propset, cmd_propdel, cmd_chexec, cmd_edit, cmd_logout,
     cmd_unknown
 /* DON'T FORGET TO ADD A NEW COMMAND ALIAS WHEN YOU ADD A NEW COMMAND */
 };
@@ -102,7 +102,7 @@ extern int in_curses;
 void init_signals(void);
 
 #ifdef HAVE_LIBREADLINE
-char *command_generator(char *text, int state);
+char *command_generator(const char *text, int state);
 #else
 /* roll our own */
 char *readline(const char *prompt);
