@@ -1,6 +1,6 @@
 /* 
    cadaver, command-line DAV client
-   Copyright (C) 1999-2001, Joe Orton <joe@orton.demon.co.uk>, 
+   Copyright (C) 1999-2001, Joe Orton <joe@manyfish.co.uk>, 
    except where otherwise indicated.
 
    This program is free software; you can redistribute it and/or modify
@@ -26,7 +26,7 @@
  *     link or a loaded server, a request can take AGES to return.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <sys/types.h>
 
@@ -75,6 +75,7 @@
 #include "cadaver.h"
 #include "commands.h"
 #include "options.h"
+#include "utils.h"
 
 int yesno(void);
 
@@ -92,7 +93,7 @@ const static struct {
     C(pwd), C(help), C(put), C(get), C(mkcol), C(delete), C(move), C(copy),
     C(less), C(cat), C(lpwd), C(lcd), C(lls), C(echo), C(quit),
     C(mget), C(mput), C(rmcol), C(lock), C(unlock), C(discover), C(steal),
-    C(chexec), C(showlocks), C(version), C(propget), C(propset),
+    C(chexec), C(showlocks), C(version), C(propget), C(propset), C(propdel),
 #if 0
 C(propedit), 
 #endif
@@ -187,13 +188,11 @@ static void dispatch(const char *verb, const char *filename,
 
 static void display_help_message(void);
 
-static void print_lock(const struct ne_lock *lock);
-
 #ifdef HAVE_LIBREADLINE
 
 /* Command name generator for readline.
  * Copied almost verbatim from the info doc */
-char *command_generator(char *text, int state)
+char *command_generator(const char *text, int state)
 {
     static int i, len;
     const char *name;
@@ -336,7 +335,7 @@ static char *true_path(const char *res)
 {
     char *full;
     full = resolve_path(path, res, false);
-    if (is_collection(full)) {
+    if (getrestype(full) == resr_collection) {
 	if (!uri_has_trailing_slash(full)) {
 	    char *tmp;
 	    CONCAT2(tmp, full, "/");
@@ -365,14 +364,14 @@ static const char *get_locktype(enum ne_lock_type t)
     }
 }
 
-static const char *get_timeout(int t)
+static const char *get_timeout(long t)
 {
     static char buf[128];
     switch (t) {
     case NE_TIMEOUT_INFINITE: return _("infinite");
     case NE_TIMEOUT_INVALID: return _("invalid");
     default:
-	sprintf(buf, "%d", t);
+	sprintf(buf, _("%ld seconds"), t);
 	return buf;
     }
 }
@@ -395,19 +394,24 @@ static const char *get_depth(int d)
 static void print_lock(const struct ne_lock *lock)
 {
     printf(_("Lock <%s> on %s:\n"
-	     "  Scope: %s  Type: %s  Timeout: %s\n"
-	     "  Owner: %s  Depth: %s\n"), 
+	     "  Scope: %s  Type: %s  Timeout: %s  Depth: %s\n"
+	     "  Owner: %s\n"), 
 	       lock->token, lock->uri,
 	   get_lockscope(lock->scope),
 	   get_locktype(lock->type), get_timeout(lock->timeout),
-	   lock->owner?lock->owner:_("(none)"), get_depth(lock->depth));
+	   get_depth(lock->depth), lock->owner?lock->owner:_("(none)"));
 }
 
 static void discover_result(void *userdata, const struct ne_lock *lock, 
 			    const char *uri, const ne_status *status)
 {
+    int *count = userdata;
     if (lock) {
+	if (*count == 0) {
+	    printf("\n");
+	}
 	print_lock(lock);
+	*count += 1;
     } else {
 	printf(_("Failed on %s: %d %s\n"), uri, 
 	       status->code, status->reason_phrase);
@@ -417,14 +421,15 @@ static void discover_result(void *userdata, const struct ne_lock *lock,
 void execute_discover(const char *res)
 {
     char *real_remote;
-    int ret;
+    int ret, count = 0;
     real_remote = resolve_path(path, res, false);
     out_start(_("Discovering locks on"), res);
-    ret = ne_lock_discover(session, real_remote, 
-			    discover_result, NULL);
+    ret = ne_lock_discover(session, real_remote, discover_result, &count);
     switch (ret) {
     case NE_OK:
-	out_success();
+	if (count == 0) {
+	    printf(" no locks found.\n");
+	}
 	break;
     default:
 	out_result(ret);
@@ -436,9 +441,14 @@ void execute_discover(const char *res)
 static void steal_result(void *userdata, const struct ne_lock *lock, 
 			 const char *uri, const ne_status *status)
 {
+    int *count = userdata;
     if (lock != NULL) {
+	if (*count == 0) {
+	    printf("\n");
+	}
 	printf("%s: <%s>\n", lock->uri, lock->token);
 	ne_lock_add(lock_session, ne_lock_copy(lock));
+	*count += 1;
     } else {
 	printf(_("Failed on %s: %d %s\n"), uri,
 	       status->code, status->reason_phrase);
@@ -448,14 +458,15 @@ static void steal_result(void *userdata, const struct ne_lock *lock,
 void execute_steal(const char *res)
 {
     char *real_remote;
-    int ret;
+    int ret, count = 0;
     real_remote = resolve_path(path, res, false);
     out_start(_("Stealing locks on"), res);
-    ret = ne_lock_discover(session, real_remote, 
-			    steal_result, NULL);
+    ret = ne_lock_discover(session, real_remote, steal_result, &count);
     switch (ret) {
     case NE_OK:
-	out_success();
+	if (count == 0) {
+	    printf(" no locks found.\n");
+	}
 	break;
     default:
 	out_result(ret);
@@ -482,16 +493,31 @@ void execute_lock(const char *res)
 {
     char *real_remote;
     struct ne_lock *lock;
-    lock = ne_calloc(sizeof *lock);
-    out_start(_("Locking"), res);
-    real_remote = true_path(res);
-    lock->type = ne_locktype_write;
-    lock->scope = ne_lockscope_exclusive;
-    lock->owner = getowner();
-    if (uri_has_trailing_slash(real_remote)) {
-	lock->depth = NE_DEPTH_INFINITE;
+    enum resource_type rtype;
+
+    real_remote = resolve_path(path, res, false);
+    rtype = getrestype(real_remote);
+    if (rtype == resr_collection) {
+	if (!uri_has_trailing_slash(real_remote)) {
+	    char *tmp;
+	    CONCAT2(tmp, real_remote, "/");
+	    free(real_remote);
+	    real_remote = tmp;
+	}
+	out_start(_("Locking collection"), res);
     } else {
-	lock->depth = 0;
+	out_start(_("Locking"), res);
+    }
+    lock = ne_calloc(sizeof *lock);
+    lock->type = ne_locktype_write;
+    lock->scope = lockscope;
+    lock->owner = getowner();
+    if (rtype == resr_normal) {
+	/* for non-cols, only zero makes sense */
+	lock->depth = NE_DEPTH_ZERO;
+    } else {
+	/* use value of lockdepth option. */
+	lock->depth = lockdepth;
     }
     lock->uri = real_remote;
     if (out_handle(ne_lock(session,lock))) {
@@ -590,29 +616,47 @@ static void pget_results(void *userdata, const char *uri,
     free(dname);
 }
 
-void execute_propset(const char *res, const char *name, const char *value)
+/* Change property 'name' on 'uri': if value is non-NULL, set property
+ * to have new value, else delete it. */
+static void propop(const char *descr, const char *res, 
+		   const char *name, const char *value)
 {
     ne_proppatch_operation ops[2];
     ne_propname pname;
     char *uri = resolve_path(path, res, false);
-    char *val, *encname = utf8_encode(name);
+    char *val = NULL /* quieten gcc */, *encname = utf8_encode(name);
 
     ops[0].name = &pname;
-    ops[0].type = ne_propset;
-    ops[0].value = val = utf8_encode(value);
+    if (value) {
+	ops[0].type = ne_propset;
+	ops[0].value = val = utf8_encode(value);
+    } else {
+	ops[0].type = ne_propremove;
+    }
     ops[1].name = NULL;
     
     pname.nspace = (const char *)get_option(opt_namespace);
     pname.name = encname;
-
-    out_start(_("Setting property on "), res);
+    out_start(descr, res);
     out_handle(ne_proppatch(session, uri, ops));
-    
-    free(val);
+
+    if (value) 
+	free(val);
     free(encname);
     free(uri);
 }    
 
+
+void execute_propset(const char *res, const char *name, const char *value)
+{
+    propop(_("Setting property on"), res, name, value);
+}
+
+void execute_propdel(const char *res, const char *name)
+{
+    propop(_("Deleting property on"), res, name, NULL);
+}
+
 void execute_propget(const char *res, const char *name)
 {
     ne_propname pnames[2] = {{NULL}, {NULL}};
@@ -681,14 +725,14 @@ void execute_delete(const char *filename)
 {
     char *remote = resolve_path(path, filename, false);
     out_start(_("Deleting"), filename);
-    if (is_collection(remote)) {
+    if (getrestype(remote) == resr_collection) {
 	output(o_finish, 
 _("is a collection resource.\n"
 "The `rm' command cannot be used to delete a collection.\n"
 "Use `rmcol %s' to delete this collection and ALL its contents.\n"), 
 filename);
     } else {
-	if (out_handle(ne_delete(session, remote))) {
+	if (!out_handle(ne_delete(session, remote))) {
 	    remove_locks(remote, 0);
 	}
     }
@@ -700,7 +744,7 @@ void execute_rmcol(const char *filename)
     char *remote;
     remote = resolve_path(path, filename, true);
     out_start(_("Deleting collection"), filename);
-    if (!is_collection(remote)) {
+    if (getrestype(remote) != resr_collection) {
 	output(o_finish, 
 	       _("is not a collection.\n"
 		 "The `rmcol' command can only be used to delete collections.\n"
@@ -806,33 +850,6 @@ void execute_cat(const char *resource)
     }
 }
 
-/* Returns non-zero if given resource is not a collection resource.
- * This function MAY make a request to the server. */
-int is_collection(const char *uri)
-{
-    struct resource *res = NULL;
-    int ret = 0;
-    /* TODO: just request resourcetype here. */
-    ret = fetch_resource_list(session, uri, NE_DEPTH_ZERO, 1, &res);
-    if (ret == NE_OK) {
-	if (res != NULL && uri_compare(uri, res->uri) == 0) {
-	    if (res->type == resr_collection) {
-		ret = 1;
-	    } else {
-		ne_set_error(session, _("Not a collection resource."));
-	    }
-	} else {
-	    /* FIXME: this error occurs when you do open /foo and get
-	     * the response for /foo/ back */
-	    ne_set_error(session, _("Did not find a collection resource."));
-	}
-    } else {
-	ret = 0;
-    }
-    free_resource_list(res);
-    return ret;
-}
-
 void multi_copy(int argc, const char *argv[])
 {
     do_copymove(argc, argv, _("copying"), _("copy"), simple_copy);
@@ -851,7 +868,7 @@ static void do_copymove(int argc, const char *argv[],
     char *dest;
 
     dest = resolve_path(path, argv[argc-1], true);
-    if (is_collection(dest)) {
+    if (getrestype(dest) == resr_collection) {
 	int n;
 	char *real_src, *real_dest;
 	for(n = 0; n < argc-1; n++) {
@@ -884,13 +901,14 @@ static void do_copymove(int argc, const char *argv[],
 static void simple_move(const char *src, const char *dest) 
 {
     output(o_start, _("Moving `%s' to `%s': "), src, dest);
-    out_result(ne_move(session, 1, src, dest));
+    out_result(ne_move(session, get_bool_option(opt_overwrite), src, dest));
 }
 
 static void simple_copy(const char *src, const char *dest) 
 {
     output(o_start, _("Copying `%s' to `%s': "), src, dest);
-    out_result(ne_copy(session, 1, NE_DEPTH_INFINITE, src, dest));
+    out_result(ne_copy(session, get_bool_option(opt_overwrite), 
+		       NE_DEPTH_INFINITE, src, dest));
 }
 
 /* TODO: can do utf-8 handling here. */
@@ -973,7 +991,7 @@ void execute_get(const char *remote, const char *local)
 	filename = ne_strdup(local);
     }
     {
-	int fd = open(filename, O_CREAT|O_WRONLY);
+	int fd = open(filename, O_CREAT|O_WRONLY, 0644);
 	output(o_transfer, _("Downloading `%s' to %s:"), real_remote, filename);
 	if (fd < 0) {
 	    output(o_finish, _("failed:\n%s\n"), strerror(errno));
@@ -1056,7 +1074,7 @@ void execute_edit(const char *remote)
      * property, and edit on that instead: IIS does expose such a
      * property. Would be a nice hack to add the support to mod_dav
      * too. */
-    if (is_collection(real_remote)) {
+    if (getrestype(real_remote) == resr_collection) {
 	printf(_("You cannot edit a collection resource (%s).\n"),
 	       real_remote);
 	goto edit_bail;
