@@ -21,7 +21,7 @@
 #ifndef COMMON_H
 #define COMMON_H
 
-#include <config.h>
+#include "config.h"
 
 #include <sys/types.h>
 
