@@ -1,6 +1,6 @@
 /* 
    cadaver, command-line DAV client
-   Copyright (C) 1999-2001, Joe Orton <joe@orton.demon.co.uk>
+   Copyright (C) 1999-2001, Joe Orton <joe@manyfish.co.uk>
                                                                      
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 
 /* Options handling */
 
-#include <config.h>
+#include "config.h"
 
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
@@ -34,6 +34,7 @@
 #include <ne_request.h>
 #include <ne_utils.h>
 #include <ne_alloc.h>
+#include <ne_basic.h> /* NE_DEPTH_* */
 
 #include "common.h"
 #include "cadaver.h"
@@ -43,9 +44,20 @@ static void set_debug(const char *new);
 static void unset_debug(const char *new);
 static void disp_debug(void);
 
+static void set_lockscope(const char *new);
+static void unset_lockscope(const char *new);
+static void disp_lockscope(void);
+
+static void set_lockdepth(const char *new);
+static void unset_lockdepth(const char *new);
+static void disp_lockdepth(void);
+
 /* Option holders */
 
-static int enable_expect, presume_utf8;
+static int enable_expect, presume_utf8, overwrite, quiet;
+
+enum ne_lock_scope lockscope;
+int lockdepth;
 
 static struct {
     const char *name;
@@ -68,8 +80,10 @@ static struct {
 #define B(x,v,h) { #x, opt_##x, v, opt_bool, NULL, NULL, NULL, h, NULL }
     /* Booleans */
     B(tolerant, &tolerant, "Tolerate non-WebDAV collections"),
+    B(overwrite, &overwrite, "Enable overwrite (e.g. on copy/move operations"),
     B(expect100, &enable_expect, "Enable use of 'Expect: 100-continue' header"),
     B(utf8, &presume_utf8, "Presume filenames etc are UTF-8 encoded"),
+    B(quiet, &quiet, "Whether to display connection status messages"),
 #undef B
 #define S(x,h) { #x, opt_##x, NULL, opt_string, NULL, NULL, NULL, h, NULL }
     S(lockowner, "Lock owner URI"),
@@ -86,6 +100,14 @@ static struct {
       "The debug value is a list of comma-separated keywords.\n"
       "Valid keywords are: socket, http, xml, httpauth, cleartext."
     },
+    { "lockscope", opt_lockscope, NULL, opt_handled,
+      set_lockscope, unset_lockscope, disp_lockscope, "Lock scope options",
+      "The lockscope value must be one of two valid keywords: exclusive or shared."
+    },
+    { "lockdepth", opt_lockdepth, NULL, opt_handled,
+      set_lockdepth, unset_lockdepth, disp_lockdepth, "Lock depth options",
+      "The lockdepth value must be 0 or infinity."
+    },
     { NULL, 0 }
 };
 
@@ -187,6 +209,58 @@ static void disp_debug(void)
     putchar('}');
 }
 
+static void set_lockscope(const char *set)
+{
+    if (strcasecmp(set,"exclusive") == 0)
+	lockscope = ne_lockscope_exclusive;
+    else if (strcasecmp(set,"shared") == 0)
+	lockscope = ne_lockscope_shared;
+    else
+	printf("Invalid value for lockscope. Try `set lockscope' for more info.\n");
+}
+
+static void unset_lockscope(const char *s)
+{
+    lockscope = ne_lockscope_exclusive;
+}
+
+static void disp_lockscope(void)
+{
+    if (lockscope == ne_lockscope_exclusive)
+	printf("exclusive");
+    else if (lockscope == ne_lockscope_shared)
+	printf("shared");
+    else
+	printf("illegal value");
+}
+
+static void set_lockdepth(const char *set)
+{
+    if (strcmp(set, "0") == 0 ||
+	strcasecmp(set, "zero") == 0)
+	lockdepth = NE_DEPTH_ZERO;
+    else if (strcasecmp(set, "infinite") == 0 ||
+	     strcasecmp(set, "infinity") == 0)
+	lockdepth = NE_DEPTH_INFINITE;
+    else
+	printf("Invalid value for lockdepth. Try `set lockdepth' for more info.\n");
+}
+
+static void unset_lockdepth(const char *s)
+{
+    lockdepth = NE_DEPTH_INFINITE;
+}
+
+static void disp_lockdepth(void)
+{
+    if (lockdepth == NE_DEPTH_ZERO)
+	printf("zero");
+    else if (lockdepth == NE_DEPTH_INFINITE)
+	printf("infinite");
+    else
+	printf("illegal value");
+}
+
 void execute_set(const char *opt, const char *newv)
 {
     if (opt == NULL) {
@@ -280,6 +354,12 @@ int get_bool_option(enum option_id id)
     return *ret;
 }
 
+void set_bool_option(enum option_id id, int truth)
+{
+    int *opt = get_option(id);
+    *opt = truth;
+}
+
 void set_option(enum option_id id, void *newval)
 {
     int n;
