@@ -20,6 +20,8 @@
 #ifndef OPTIONS_H
 #define OPTIONS_H
 
+#include <ne_locks.h>
+
 enum option_id {
     opt_tolerant,
     opt_expect100,
@@ -28,12 +30,19 @@ enum option_id {
     opt_cert,
     opt_certkey,
     opt_namespace,
+    opt_quiet,
     opt_proxy,
     opt_proxy_port,
     opt_debug,
-    opt_utf8
+    opt_utf8,
+    opt_overwrite,
+    opt_lockdepth,
+    opt_lockscope
 };
 
+extern int lockdepth; /* current lock depth setting. */
+extern enum ne_lock_scope lockscope; /* current lock scope setting. */
+
 void execute_set( const char *opt, const char * );
 void execute_unset( const char *opt, const char * );
 
@@ -41,5 +50,6 @@ void *get_option( enum option_id id );
 void set_option( enum option_id id, void *newval );
 
 int get_bool_option(enum option_id id);
+void set_bool_option(enum option_id id, int truth);
 
 #endif /* OPTIONS_H */
